@@ -101,7 +101,9 @@ my %banfunc = (
     "open" => 1,
     "printf" => 1,
     "realloc" => 1,
+    "recv" => 1,
     "rename" => 1,
+    "send" => 1,
     "snprintf" => 1,
     "socket" => 1,
     "socketpair" => 1,
